
#ifndef MIMETREEPARSER_CORE_EXPORT_H
#define MIMETREEPARSER_CORE_EXPORT_H

#ifdef MIMETREEPARSER_CORE_STATIC_DEFINE
#  define MIMETREEPARSER_CORE_EXPORT
#  define MIMETREEPARSER_CORE_NO_EXPORT
#else
#  ifndef MIMETREEPARSER_CORE_EXPORT
#    ifdef KPim6MimeTreeParserCore_EXPORTS
        /* We are building this library */
#      define MIMETREEPARSER_CORE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define MIMETREEPARSER_CORE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef MIMETREEPARSER_CORE_NO_EXPORT
#    define MIMETREEPARSER_CORE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef MIMETREEPARSER_CORE_DEPRECATED
#  define MIMETREEPARSER_CORE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef MIMETREEPARSER_CORE_DEPRECATED_EXPORT
#  define MIMETREEPARSER_CORE_DEPRECATED_EXPORT MIMETREEPARSER_CORE_EXPORT MIMETREEPARSER_CORE_DEPRECATED
#endif

#ifndef MIMETREEPARSER_CORE_DEPRECATED_NO_EXPORT
#  define MIMETREEPARSER_CORE_DEPRECATED_NO_EXPORT MIMETREEPARSER_CORE_NO_EXPORT MIMETREEPARSER_CORE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef MIMETREEPARSER_CORE_NO_DEPRECATED
#    define MIMETREEPARSER_CORE_NO_DEPRECATED
#  endif
#endif

#endif /* MIMETREEPARSER_CORE_EXPORT_H */
